"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:vdb' }"

"{ NameSpace: Smalltalk }"

Object subclass:#VDBVirtualMemoryMap
	instanceVariableNames:'regions'
	classVariableNames:''
	poolDictionaries:''
	category:'VDB-Tools'
!

!VDBVirtualMemoryMap class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!VDBVirtualMemoryMap class methodsFor:'instance creation'!

fromSysInternalsVmmapDump: aStringOrFilenameOrStream
    | s |

    s := aStringOrFilenameOrStream readStream.
    ^ [
        self new initializeFromSysInternalsVmmapDump: s
    ] ensure:[ 
        s close.
    ].

    "Created: / 13-10-2017 / 12:11:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBVirtualMemoryMap methodsFor:'reading'!

initializeFromSysInternalsVmmapDump: aStream
    "
    Initializes regions from SysInterna's vmmap.exe dump (.mmp)
    "
    | dump snapshot |

    dump := XML::XMLParser parse: aStream.
    snapshot := (dump / 'root' / 'Snapshots' / 'Snapshot') last.
    regions := (snapshot / 'MemoryRegions' / 'Region') 
                select:[ :node | 
                    (node @ 'Commit') ~= '0'.
                ]
                thenCollect:[ :node |
                    | region |

                    region := VDBVirtualMemoryRegion new
                                address:     (node @ 'Address') asInteger;
                                size:        (node @ 'Size') asInteger;
                                protection:  (node @ 'Protection') asInteger;
                                comment:     (node @ 'Details');
                                yourself.
                ] as: OrderedCollection.

    regions sort:[ :a :b | a address  < b address ].

    "Created: / 13-10-2017 / 12:06:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

